@@ -18,7 +18,7 @@ require "highline/color_scheme"
 require "highline/style"
 
 #
-# A HighLine object is a "high-level line oriented" shell over an input and an 
+# A HighLine object is a "high-level line oriented" shell over an input and an
 # output stream.  HighLine simplifies common console interaction, effectively
 # replacing puts() and gets().  User code can simply specify the question to ask
 # and any details about user interaction, then leave the rest of the work to
@@ -28,41 +28,41 @@ require "highline/style"
 #
 class HighLine
   # The version of the installed library.
-  VERSION = "1.6.13".freeze
-  
+  VERSION = "1.6.20".freeze
+
   # An internal HighLine error.  User code does not need to trap this.
   class QuestionError < StandardError
     # do nothing, just creating a unique error type
   end
-  
+
   # The setting used to disable color output.
   @@use_color = true
-  
+
   # Pass +false+ to _setting_ to turn off HighLine's color escapes.
   def self.use_color=( setting )
     @@use_color = setting
   end
-  
+
   # Returns true if HighLine is currently using color escapes.
   def self.use_color?
     @@use_color
   end
-  
+
   # For checking if the current version of HighLine supports RGB colors
   # Usage: HighLine.supports_rgb_color? rescue false   # rescue for compatibility with older versions
   # Note: color usage also depends on HighLine.use_color being set
   def self.supports_rgb_color?
     true
   end
-  
+
   # The setting used to disable EOF tracking.
   @@track_eof = true
-  
+
   # Pass +false+ to _setting_ to turn off HighLine's EOF tracking.
   def self.track_eof=( setting )
     @@track_eof = setting
   end
-  
+
   # Returns true if HighLine is currently tracking EOF for input.
   def self.track_eof?
     @@track_eof
@@ -71,7 +71,7 @@ class HighLine
   # The setting used to control color schemes.
   @@color_scheme = nil
 
-  # Pass ColorScheme to _setting_ to turn set a HighLine color scheme.
+  # Pass ColorScheme to _setting_ to set a HighLine color scheme.
   def self.color_scheme=( setting )
     @@color_scheme = setting
   end
@@ -87,13 +87,13 @@ class HighLine
   end
 
   #
-  # Embed in a String to clear all previous ANSI sequences.  This *MUST* be 
+  # Embed in a String to clear all previous ANSI sequences.  This *MUST* be
   # done before the program exits!
   #
-  
+
   ERASE_LINE_STYLE = Style.new(:name=>:erase_line, :builtin=>true, :code=>"\e[K")  # Erase the current line of terminal output
   ERASE_CHAR_STYLE = Style.new(:name=>:erase_char, :builtin=>true, :code=>"\e[P")  # Erase the character under the cursor.
-  CLEAR_STYLE      = Style.new(:name=>:clear,      :builtin=>true, :code=>"\e[0m") # Clear color settings 
+  CLEAR_STYLE      = Style.new(:name=>:clear,      :builtin=>true, :code=>"\e[0m") # Clear color settings
   RESET_STYLE      = Style.new(:name=>:reset,      :builtin=>true, :code=>"\e[0m") # Alias for CLEAR.
   BOLD_STYLE       = Style.new(:name=>:bold,       :builtin=>true, :code=>"\e[1m") # Bold; Note: bold + a color works as you'd expect,
                                                               # for example bold black. Bold without a color displays
@@ -104,7 +104,7 @@ class HighLine
   BLINK_STYLE      = Style.new(:name=>:blink,      :builtin=>true, :code=>"\e[5m") # Blink; support uncommon
   REVERSE_STYLE    = Style.new(:name=>:reverse,    :builtin=>true, :code=>"\e[7m") # Reverse foreground and background
   CONCEALED_STYLE  = Style.new(:name=>:concealed,  :builtin=>true, :code=>"\e[8m") # Concealed; support uncommon
-  
+
   STYLES = %w{CLEAR RESET BOLD DARK UNDERLINE UNDERSCORE BLINK REVERSE CONCEALED}
 
   # These RGB colors are approximate; see http://en.wikipedia.org/wiki/ANSI_escape_code
@@ -116,15 +116,15 @@ class HighLine
   MAGENTA_STYLE    = Style.new(:name=>:magenta,    :builtin=>true, :code=>"\e[35m", :rgb=>[128,  0,128])
   CYAN_STYLE       = Style.new(:name=>:cyan,       :builtin=>true, :code=>"\e[36m", :rgb=>[  0,128,128])
   # On Mac OSX Terminal, white is actually gray
-  WHITE_STYLE      = Style.new(:name=>:white,      :builtin=>true, :code=>"\e[37m", :rgb=>[192,192,192]) 
+  WHITE_STYLE      = Style.new(:name=>:white,      :builtin=>true, :code=>"\e[37m", :rgb=>[192,192,192])
   # Alias for WHITE, since WHITE is actually a light gray on Macs
   GRAY_STYLE       = Style.new(:name=>:gray,       :builtin=>true, :code=>"\e[37m", :rgb=>[192,192,192])
-  # On Mac OSX Terminal, this is black foreground, or bright white background. 
+  # On Mac OSX Terminal, this is black foreground, or bright white background.
   # Also used as base for RGB colors, if available
-  NONE_STYLE       = Style.new(:name=>:none,       :builtin=>true, :code=>"\e[38m", :rgb=>[  0,  0,  0]) 
-  
+  NONE_STYLE       = Style.new(:name=>:none,       :builtin=>true, :code=>"\e[38m", :rgb=>[  0,  0,  0])
+
   BASIC_COLORS = %w{BLACK RED GREEN YELLOW BLUE MAGENTA CYAN WHITE GRAY NONE}
-  
+
   colors = BASIC_COLORS.dup
   BASIC_COLORS.each do |color|
     bright_color = "BRIGHT_#{color}"
@@ -132,18 +132,18 @@ class HighLine
     const_set bright_color+'_STYLE', const_get(color + '_STYLE').bright
   end
   COLORS = colors
-  
+
   colors.each do |color|
     const_set color, const_get("#{color}_STYLE").code
     const_set "ON_#{color}_STYLE", const_get("#{color}_STYLE").on
     const_set "ON_#{color}", const_get("ON_#{color}_STYLE").code
   end
   ON_NONE_STYLE.rgb = [255,255,255] # Override; white background
-  
+
   STYLES.each do |style|
     const_set style, const_get("#{style}_STYLE").code
   end
-  
+
   # For RGB colors:
   def self.const_missing(name)
     if name.to_s =~ /^(ON_)?(RGB_)([A-F0-9]{6})(_STYLE)?$/ # RGB color
@@ -174,28 +174,17 @@ class HighLine
   # and _output_.
   #
   def initialize( input = $stdin, output = $stdout,
-                  wrap_at = nil, page_at = nil )
+                  wrap_at = nil, page_at = nil, indent_size=3, indent_level=0 )
     @input   = input
     @output  = output
-    if JRUBY
-      require 'java'
-      java_import 'java.io.OutputStreamWriter'
-      java_import 'java.nio.channels.Channels'
-      java_import 'jline.ConsoleReader'
-      java_import 'jline.Terminal'
 
-      @java_input = Channels.newInputStream($stdin.to_channel)
-      @java_output = OutputStreamWriter.new(Channels.newOutputStream($stdout.to_channel))
-      @java_terminal = Terminal.getTerminal
-      @java_console = ConsoleReader.new(@java_input, @java_output)
-      @java_console.setUseHistory(false)
-      @java_console.setBellEnabled(true)
-      @java_console.setUsePagination(false)
-    end
-    
+    @multi_indent = true
+    @indent_size = indent_size
+    @indent_level = indent_level
+
     self.wrap_at = wrap_at
     self.page_at = page_at
-    
+
     @question = nil
     @answer   = nil
     @menu     = nil
@@ -204,22 +193,30 @@ class HighLine
     @gather   = nil
     @answers  = nil
     @key      = nil
+
+    initialize_system_extensions if respond_to?(:initialize_system_extensions)
   end
-  
+
   include HighLine::SystemExtensions
-  
+
   # The current column setting for wrapping output.
   attr_reader :wrap_at
   # The current row setting for paging output.
   attr_reader :page_at
-  
+  # Indentation over multiple lines
+  attr_accessor :multi_indent
+  # The indentation size
+  attr_accessor :indent_size
+  # The indentation level
+  attr_accessor :indent_level
+
   #
   # A shortcut to HighLine.ask() a question that only accepts "yes" or "no"
   # answers ("y" and "n" are allowed) and returns +true+ or +false+
   # (+true+ for "yes").  If provided a +true+ value, _character_ will cause
   # HighLine to fetch a single character response. A block can be provided
   # to further configure the question as in HighLine.ask()
-  # 
+  #
   # Raises EOFError if input is exhausted.
   #
   def agree( yes_or_no_question, character = nil )
@@ -228,11 +225,11 @@ class HighLine
       q.responses[:not_valid]    = 'Please enter "yes" or "no".'
       q.responses[:ask_on_error] = :question
       q.character                = character
-      
+
       yield q if block_given?
     end
   end
-  
+
   #
   # This method is the primary interface for user input.  Just provide a
   # _question_ to ask the user, the _answer_type_ you want returned, and
@@ -240,36 +237,39 @@ class HighLine
   # handled.  See HighLine.say() for details on the format of _question_, and
   # HighLine::Question for more information about _answer_type_ and what's
   # valid in the code block.
-  # 
+  #
   # If <tt>@question</tt> is set before ask() is called, parameters are
   # ignored and that object (must be a HighLine::Question) is used to drive
   # the process instead.
-  # 
+  #
   # Raises EOFError if input is exhausted.
   #
   def ask( question, answer_type = String, &details ) # :yields: question
     @question ||= Question.new(question, answer_type, &details)
-    
+
     return gather if @question.gather
-  
-    # readline() needs to handle it's own output, but readline only supports 
-    # full line reading.  Therefore if @question.echo is anything but true, 
+
+    # readline() needs to handle its own output, but readline only supports
+    # full line reading.  Therefore if @question.echo is anything but true,
     # the prompt will not be issued. And we have to account for that now.
-    say(@question) unless (@question.readline and @question.echo == true)
+    # Also, JRuby-1.7's ConsoleReader.readLine() needs to be passed the prompt
+    # to handle line editing properly.
+    say(@question) unless ((JRUBY or @question.readline) and @question.echo == true)
+
     begin
       @answer = @question.answer_or_default(get_response)
       unless @question.valid_answer?(@answer)
         explain_error(:not_valid)
         raise QuestionError
       end
-      
+
       @answer = @question.convert(@answer)
-      
+
       if @question.in_range?(@answer)
         if @question.confirm
           # need to add a layer of scope to ask a question inside a
           # question, without destroying instance data
-          context_change = self.class.new(@input, @output, @wrap_at, @page_at)
+          context_change = self.class.new(@input, @output, @wrap_at, @page_at, @indent_size, @indent_level)
           if @question.confirm == true
             confirm_question = "Are you sure?  "
           else
@@ -283,7 +283,7 @@ class HighLine
             raise QuestionError
           end
         end
-        
+
         @answer
       else
         explain_error(:not_in_range)
@@ -295,7 +295,7 @@ class HighLine
       raise if error.is_a?(NoMethodError)
       if error.message =~ /ambiguous/
         # the assumption here is that OptionParser::Completion#complete
-        # (used for ambiguity resolution) throws exceptions containing 
+        # (used for ambiguity resolution) throws exceptions containing
         # the word 'ambiguous' whenever resolution fails
         explain_error(:ambiguous_completion)
       else
@@ -314,21 +314,23 @@ class HighLine
   # This method is HighLine's menu handler.  For simple usage, you can just
   # pass all the menu items you wish to display.  At that point, choose() will
   # build and display a menu, walk the user through selection, and return
-  # their choice amoung the provided items.  You might use this in a case
+  # their choice among the provided items.  You might use this in a case
   # statement for quick and dirty menus.
-  # 
+  #
   # However, choose() is capable of much more.  If provided, a block will be
   # passed a HighLine::Menu object to configure.  Using this method, you can
   # customize all the details of menu handling from index display, to building
   # a complete shell-like menuing system.  See HighLine::Menu for all the
   # methods it responds to.
-  # 
+  #
   # Raises EOFError if input is exhausted.
-  # 
+  #
   def choose( *items, &details )
     @menu = @question = Menu.new(&details)
     @menu.choices(*items) unless items.empty?
-    
+
+    # Set auto-completion
+    @menu.completion = @menu.options
     # Set _answer_type_ so we can double as the Question for ask().
     @menu.answer_type = if @menu.shell
       lambda do |command|    # shell-style selection
@@ -347,11 +349,11 @@ class HighLine
     else
       @menu.options          # normal menu selection, by index or name
     end
-    
+
     # Provide hooks for ERb layouts.
     @header   = @menu.header
     @prompt   = @menu.prompt
-    
+
     if @menu.shell
       selected = ask("Ignored", @menu.answer_type)
       @menu.select(self, *selected)
@@ -365,11 +367,11 @@ class HighLine
   # This method provides easy access to ANSI color sequences, without the user
   # needing to remember to CLEAR at the end of each sequence.  Just pass the
   # _string_ to color, followed by a list of _colors_ you would like it to be
-  # affected by.  The _colors_ can be HighLine class constants, or symbols 
+  # affected by.  The _colors_ can be HighLine class constants, or symbols
   # (:blue for BLUE, for example).  A CLEAR will automatically be embedded to
   # the end of the returned String.
-  # 
-  # This method returns the original _string_ unchanged if HighLine::use_color? 
+  #
+  # This method returns the original _string_ unchanged if HighLine::use_color?
   # is +false+.
   #
   def self.color( string, *colors )
@@ -381,28 +383,28 @@ class HighLine
   def self.color_code(*colors)
     Style(*colors).code
   end
-  
+
   # Works as an instance method, same as the class method
   def color_code(*colors)
     self.class.color_code(*colors)
   end
-  
+
   # Works as an instance method, same as the class method
   def color(*args)
     self.class.color(*args)
   end
-  
+
   # Remove color codes from a string
   def self.uncolor(string)
     Style.uncolor(string)
   end
-  
+
   # Works as an instance method, same as the class method
   def uncolor(string)
     self.class.uncolor(string)
   end
-  
-  # 
+
+  #
   # This method is a utility for quickly and easily laying out lists.  It can
   # be accessed within ERb replacements of any text that will be sent to the
   # user.
@@ -428,25 +430,29 @@ class HighLine
   #                                    _option_ or a default of " or ".  All
   #                                    other _items_ are separated by ", ".
   # <tt>:rows</tt>::                   The default mode.  Each of the _items_ is
-  #                                    placed on it's own line.  The _option_
+  #                                    placed on its own line.  The _option_
   #                                    parameter is ignored in this mode.
-  # 
+  #
   # Each member of the _items_ Array is passed through ERb and thus can contain
-  # their own expansions.  Color escape expansions do not contribute to the 
+  # their own expansions.  Color escape expansions do not contribute to the
   # final field width.
-  # 
+  #
   def list( items, mode = :rows, option = nil )
     items = items.to_ary.map do |item|
-      ERB.new(item, nil, "%").result(binding)
+      if item.nil?
+        ""
+      else
+        ERB.new(item, nil, "%").result(binding)
+      end
     end
-    
+
     if items.empty?
       ""
     else
       case mode
       when :inline
         option = " or " if option.nil?
-        
+
         if items.size == 1
           items.first
         else
@@ -467,7 +473,7 @@ class HighLine
           "%-#{pad}s" % item
         end
         row_count = (items.size / option.to_f).ceil
-      
+
         if mode == :columns_across
           rows = Array.new(row_count) { Array.new }
           items.each_with_index do |item, index|
@@ -480,7 +486,7 @@ class HighLine
           items.each_with_index do |item, index|
             columns[index / row_count] << item
           end
-      
+
           list = ""
           columns.first.size.times do |index|
             list << columns.map { |column| column[index] }.
@@ -596,79 +602,130 @@ class HighLine
       end
     end
   end
-  
+
   #
   # The basic output method for HighLine objects.  If the provided _statement_
   # ends with a space or tab character, a newline will not be appended (output
   # will be flush()ed).  All other cases are passed straight to Kernel.puts().
   #
   # The _statement_ parameter is processed as an ERb template, supporting
-  # embedded Ruby code.  The template is evaluated with a binding inside 
+  # embedded Ruby code.  The template is evaluated with a binding inside
   # the HighLine instance, providing easy access to the ANSI color constants
   # and the HighLine.color() method.
   #
   def say( statement )
-    statement = statement.to_str
+    statement = format_statement(statement)
     return unless statement.length > 0
-    
-    template  = ERB.new(statement, nil, "%")
-    statement = template.result(binding)
-    
-    statement = wrap(statement) unless @wrap_at.nil?
-    statement = page_print(statement) unless @page_at.nil?
-    
-    if statement[-1, 1] == " " or statement[-1, 1] == "\t"
-      @output.print(statement)
-      @output.flush  
+
+      # Don't add a newline if statement ends with whitespace, OR
+    # if statement ends with whitespace before a color escape code.
+    if /[ \t](\e\[\d+(;\d+)*m)?\Z/ =~ statement
+      @output.print(indentation+statement)
+      @output.flush
     else
-      @output.puts(statement)
+      @output.puts(indentation+statement)
     end
   end
-  
+
   #
   # Set to an integer value to cause HighLine to wrap output lines at the
   # indicated character limit.  When +nil+, the default, no wrapping occurs.  If
-  # set to <tt>:auto</tt>, HighLine will attempt to determing the columns
+  # set to <tt>:auto</tt>, HighLine will attempt to determine the columns
   # available for the <tt>@output</tt> or use a sensible default.
   #
   def wrap_at=( setting )
     @wrap_at = setting == :auto ? output_cols : setting
   end
-  
+
   #
   # Set to an integer value to cause HighLine to page output lines over the
   # indicated line limit.  When +nil+, the default, no paging occurs.  If
-  # set to <tt>:auto</tt>, HighLine will attempt to determing the rows available
+  # set to <tt>:auto</tt>, HighLine will attempt to determine the rows available
   # for the <tt>@output</tt> or use a sensible default.
   #
   def page_at=( setting )
     @page_at = setting == :auto ? output_rows - 2 : setting
   end
-  
-  # 
+
+  #
+  # Outputs indentation with current settings
+  #
+  def indentation
+    return ' '*@indent_size*@indent_level
+  end
+
+  #
+  # Executes block or outputs statement with indentation
+  #
+  def indent(increase=1, statement=nil, multiline=nil)
+    @indent_level += increase
+    multi = @multi_indent
+    @multi_indent = multiline unless multiline.nil?
+    begin
+        if block_given?
+            yield self
+        else
+            say(statement)
+        end
+    rescue
+        @multi_indent = multi
+        @indent_level -= increase
+        raise
+    end
+    @multi_indent = multi
+    @indent_level -= increase
+  end
+
+  #
+  # Outputs newline
+  #
+  def newline
+    @output.puts
+  end
+
+  #
   # Returns the number of columns for the console, or a default it they cannot
   # be determined.
-  # 
+  #
   def output_cols
     return 80 unless @output.tty?
     terminal_size.first
   rescue
     return 80
   end
-  
-  # 
+
+  #
   # Returns the number of rows for the console, or a default if they cannot be
   # determined.
-  # 
+  #
   def output_rows
     return 24 unless @output.tty?
     terminal_size.last
   rescue
     return 24
   end
-  
+
   private
-  
+
+  def format_statement statement
+    statement = statement.dup.to_str
+    return statement unless statement.length > 0
+
+  # Allow non-ascii menu prompts in ruby > 1.9.2. ERB eval the menu statement
+    # with the environment's default encoding(usually utf8)
+    statement.force_encoding(Encoding.default_external) if defined?(Encoding) && Encoding.default_external
+
+    template  = ERB.new(statement, nil, "%")
+    statement = template.result(binding)
+
+    statement = wrap(statement) unless @wrap_at.nil?
+    statement = page_print(statement) unless @page_at.nil?
+
+    statement = statement.gsub(/\n(?!$)/,"\n#{indentation}") if @multi_indent
+
+    statement
+  end
+
   #
   # A helper method for sending the output stream and error and repeat
   # of the question.
@@ -681,13 +738,13 @@ class HighLine
       say(@question.responses[:ask_on_error])
     end
   end
-  
+
   #
-  # Collects an Array/Hash full of answers as described in 
+  # Collects an Array/Hash full of answers as described in
   # HighLine::Question.gather().
-  # 
+  #
   # Raises EOFError if input is exhausted.
-  # 
+  #
   def gather(  )
     original_question = @question
     original_question_string = @question.question
@@ -756,10 +813,10 @@ class HighLine
   #
   # Read a line of input from the input stream and process whitespace as
   # requested by the Question object.
-  # 
+  #
   # If Question's _readline_ property is set, that library will be used to
   # fetch input.  *WARNING*:  This ignores the currently set input stream.
-  # 
+  #
   # Raises EOFError if input is exhausted.
   #
   def get_line(  )
@@ -772,12 +829,12 @@ class HighLine
       say(@question)
       question = @output.string
       @output  = old_output
-      
+
       # prep auto-completion
       Readline.completion_proc = lambda do |string|
         @question.selection.grep(/\A#{Regexp.escape(string)}/)
       end
-      
+
       # work-around ugly readline() warnings
       old_verbose = $VERBOSE
       $VERBOSE    = nil
@@ -796,13 +853,11 @@ class HighLine
       answer
     else
       if JRUBY
-        enable_echo_afterwards = @java_terminal.isEchoEnabled
-        @java_terminal.disableEcho
-        begin
-          raw_answer = @java_console.readLine(nil, nil)
-        ensure
-          @java_terminal.enableEcho if enable_echo_afterwards
-        end
+        statement = format_statement(@question)
+        raw_answer = @java_console.readLine(statement, nil)
+
+        raise EOFError, "The input stream is exhausted." if raw_answer.nil? and
+                                                            @@track_eof
       else
         raise EOFError, "The input stream is exhausted." if @@track_eof and
                                                             @input.eof?
@@ -813,46 +868,29 @@ class HighLine
     end
   end
 
-  def get_single_character(is_stty)
-    if JRUBY
-      @java_console.readVirtualKey
-    elsif is_stty
-      @input.getbyte
-    else
-      get_character(@input)
-    end
-  end
-
   #
   # Return a line or character of input, as requested for this question.
   # Character input will be returned as a single character String,
   # not an Integer.
-  # 
+  #
   # This question's _first_answer_ will be returned instead of input, if set.
-  # 
+  #
   # Raises EOFError if input is exhausted.
   #
   def get_response(  )
     return @question.first_answer if @question.first_answer?
 
-    stty = (CHARACTER_MODE == "stty")
-
     if @question.character.nil?
       if @question.echo == true and @question.limit.nil?
         get_line
       else
-        if JRUBY
-          enable_echo_afterwards = @java_terminal.isEchoEnabled
-          @java_terminal.disableEcho
-        elsif stty
-          raw_no_echo_mode
-        end
+        raw_no_echo_mode
 
         line            = ""
         backspace_limit = 0
         begin
 
-          while character = get_single_character(stty)
+          while character = get_character(@input)
             # honor backspace and delete
             if character == 127 or character == 8
               line.slice!(-1, 1)
@@ -885,11 +923,7 @@ class HighLine
             break if @question.limit and line.size == @question.limit
           end
         ensure
-          if JRUBY
-            @java_terminal.enableEcho if enable_echo_afterwards
-          elsif stty
-            restore_mode
-          end
+          restore_mode
         end
         if @question.overwrite
           @output.print("\r#{HighLine.Style(:erase_line).code}")
@@ -901,15 +935,16 @@ class HighLine
         @question.change_case(@question.remove_whitespace(line))
       end
     else
-      if JRUBY
-        enable_echo_afterwards = @java_terminal.isEchoEnabled
-        @java_terminal.disableEcho
+      if JRUBY #prompt has not been shown
+        say @question
       end
+
+      raw_no_echo_mode
       begin
         if @question.character == :getc
-          response = get_single_character(true).chr
+          response = @input.getbyte.chr
         else
-          response = get_single_character(stty).chr
+          response = get_character(@input).chr
           if @question.overwrite
             @output.print("\r#{HighLine.Style(:erase_line).code}")
             @output.flush
@@ -925,22 +960,20 @@ class HighLine
           end
         end
       ensure
-        if JRUBY
-          @java_terminal.enableEcho if enable_echo_afterwards
-        end
+        restore_mode
       end
       @question.change_case(response)
     end
   end
-  
-  # 
+
+  #
   # Page print a series of at most _page_at_ lines for _output_.  After each
   # page is printed, HighLine will pause until the user presses enter/return
   # then display the next page of data.
   #
   # Note that the final page of _output_ is *not* printed, but returned
   # instead.  This is to support any special handling for the final sequence.
-  # 
+  #
   def page_print( output )
     lines = output.scan(/[^\n]*\n?/)
     while lines.size > @page_at
@@ -951,18 +984,18 @@ class HighLine
     end
     return lines.join
   end
- 
-  # 
+
+  #
   # Ask user if they wish to continue paging output. Allows them to type "q" to
   # cancel the paging process.
-  # 
+  #
   def continue_paging?
     command = HighLine.new(@input, @output).ask(
       "-- press enter/return to continue or q to stop -- "
     ) { |q| q.character = true }
     command !~ /\A[qQ]\Z/  # Only continue paging if Q was not hit.
   end
-    
+
   #
   # Wrap a sequence of _lines_ at _wrap_at_ characters per line.  Existing
   # newlines will not be affected by this process, but additional newlines
@@ -971,30 +1004,31 @@ class HighLine
   def wrap( text )
     wrapped = [ ]
     text.each_line do |line|
-      while line =~ /([^\n]{#{@wrap_at + 1},})/
+      # take into account color escape sequences when wrapping
+      wrap_at = @wrap_at + (line.length - actual_length(line))
+      while line =~ /([^\n]{#{wrap_at + 1},})/
         search  = $1.dup
         replace = $1.dup
-        if index = replace.rindex(" ", @wrap_at)
+        if index = replace.rindex(" ", wrap_at)
           replace[index, 1] = "\n"
           replace.sub!(/\n[ \t]+/, "\n")
           line.sub!(search, replace)
         else
-          line[$~.begin(1) + @wrap_at, 0] = "\n"
+          line[$~.begin(1) + wrap_at, 0] = "\n"
         end
       end
       wrapped << line
     end
     return wrapped.join
   end
-  
-  # 
+
+  #
   # Returns the length of the passed +string_with_escapes+, minus and color
   # sequence escapes.
-  # 
+  #
   def actual_length( string_with_escapes )
     string_with_escapes.to_s.gsub(/\e\[\d{1,2}m/, "").length
   end
 end
 
 require "highline/string_extensions"
-
