@@ -14,10 +14,14 @@ class HighLine
     if JRUBY
       def initialize_system_extensions
         require 'java'
+        require 'readline'
         if JRUBY_VERSION =~ /^1.7/
           java_import 'jline.console.ConsoleReader'
 
-          @java_console = ConsoleReader.new(@input.to_inputstream, @output.to_outputstream)
+          input = @input && @input.to_inputstream
+          output = @output && @output.to_outputstream
+
+          @java_console = ConsoleReader.new(input, output)
           @java_console.set_history_enabled(false)
           @java_console.set_bell_enabled(true)
           @java_console.set_pagination_enabled(false)
@@ -176,7 +180,7 @@ class HighLine
               size = [80, 40]
               FFI::NCurses.initscr
               begin
-                size = FFI::NCurses.getmaxyx(stdscr).reverse
+                size = FFI::NCurses.getmaxyx(FFI::NCurses.stdscr).reverse
               ensure
                 FFI::NCurses.endwin
               end
